#include "simpletools.h"                      // Include simpletools   

void squareWaves3();                          // Forward declare squareWaves3
volatile int sqwPin[] = {26, 27, 5};          // 3 square wave pins
volatile int sqwFreq[] = {5, 6, 7};           // 3 square wave freqs

void cycleCount3();                           // Forward declare cycelCount3
volatile int countPin[] = {26, 27, 5};        // 3 counting pins
volatile int cycles[] = {0, 0, 0};            // 3 cycle counts

void floatCop();                              // Forward declare fp coprocessor
volatile float errorVal;                      // Float value

int main()                                    // Main function
{
  cog_run(squareWaves3, 50);                  // Run squareWave3 in other cog
  cog_run(cycleCount3, 50);                   // Run cycleCount3 in other cog                   
  cog_run(floatCop, 50);                      // Run floatCop in other cog

  int reps = 0;                               // Start reps count at 0
  while(1)                                    // Main loop
  {
    for(int i = 0; i < 3; i++)                // i 0...2
      print("cycles[%d] = %d  ",              // Display cycles counted
             i, cycles[i]);
    print("\nerrorVal = %f\n\n",              // Display errorVal
          errorVal);
    pause(1000);                              // Wait 1 s
    if(reps++ == 4)                           // If reps is 4
      for(int i = 0; i < 3; i++)              // ... double all frequencies
        sqwFreq[i] *= 2;
  }          
}

void squareWaves3()                           // Other cog 3 square waves 
{
  int f[3];                                   // For frequency tracking
  int dt[3];                                  // Time tick increments
  int t[3];                                   // Time tick tracking
  for(int i = 0; i < 3; i++)                  // Set up 3 signals
  {
    f[i] = sqwFreq[i];                        // Local copy of freqs
    low(sqwPin[i]);                           // Initialize pins
    dt[i] = CLKFREQ/(f[i]*2);                 // Set up time increments
    t[i] = CNT;                               // Mark current times
  } 
  int i = 0;                                  // Initialize i;
  while(1)                                    // Cog loop
  {
    if(f[i] != sqwFreq[i])                    // If new freq
    {
      f[i] = sqwFreq[i];                      // Update values
      dt[i] = CLKFREQ/(f[i]*2);
    }      
    if(CNT - t[i] > dt[i])                    // If time's up
    {
      toggle(sqwPin[i]);                      // Toggle pin
      t[i] += dt[i];                          // Advance toggle timer
    }      
    i++;                                      // Advance index
    if(i == 3) i = 0;                         // Reset if out of range
  }
}

void cycleCount3()                            // 3 signal counters
{
  int state[3];                               // States for comparison
  int i;
  for(i = 0; i < 3; i++)                      // Store initial states
  {
    state[i] = input(countPin[i]);            // Check each pin state
  }
  int inState;                                // For storing pin's current state
  i = 0;                                      // i -> zero
  while(1)                                    // Cog loop
  {
    inState = input(countPin[i]);             // Get input state of current pin
    if(state[i] != inState)                   // If state changed
    {
      if(!inState) cycles[i]++;               // If low, increment cycles
      state[i] = inState;                     // Remember current state
    }      
    i++;                                      // 012012012...
    if(i == 3) i = 0;
  }
}

void floatCop()                               // FP coprocessor
{
  while(1)
  {
    errorVal = (((float) cycles[1] - (float) cycles[0]) / ((float) cycles[1] + 1));
  }  
}